searchData={"items":[{"type":"module","title":"BencheeAsync","doc":"Documentation for `BencheeAsync`.","ref":"BencheeAsync.html"},{"type":"function","title":"BencheeAsync.run/2","doc":"Runs the benchmark jobs as per the Benchee-compatible configuration.\n\nFull list of configuration can be found in the [documentation for Benchee](https://hexdocs.pm/benchee/readme.html#configuration).\n\n#","ref":"BencheeAsync.html#run/2"},{"type":"function","title":"Internals - BencheeAsync.run/2","doc":"This function will inject Reporter lifecycle hooks into each bencahmark job. These hooks will be executed **before** the user provided hooks.\nThe only exception to this is where the timer reset is performed after the user's `:before_each` global hook is run.\n\nThe `BencheeAsync.Reporter` will immediately start tracking work completion on warmup end, and on job completion (where the `:after_scenario` hook is run).\nThis means that any configuration option that extends measurement times (such as `:memory_time`) will result in tracking occuring beyond the `:time` configured.","ref":"BencheeAsync.html#run/2-internals"},{"type":"module","title":"BencheeAsync.Reporter","doc":"The GenServer responsible for tracking units of work done.\n\nTo allow for more flexibility, this GenServer must be started manually before beginning the benchmarking as part of the benchmark suite setup.\n\n```elixir\n# in an .exs script\nElixir.BencheeAsync.Reporter.start_link()\n\n# in an ExUnit test\nstart_supervised!(Elixir.BencheeAsync.Reporter)\n```\n\n#","ref":"BencheeAsync.Reporter.html"},{"type":"module","title":"Reporting - BencheeAsync.Reporter","doc":"To report a unit of work done, execute `Elixir.BencheeAsync.Reporter.report/0` anywhere within your application code.\n\nTo avoid introducing dev-specific logic into your application code, it is advised to use a mocking library to mock your internal functions. [:meck](https://github.com/eproxus/meck) or [Mimic](https://github.com/edgurgel/mimic) will both work, the choice between either or otherwise would be a matter of taste.\n\nFor example, with Mimic:\n```\n# test/test_helper.exs\nMimic.copy(MyApp)\nExUnit.start()\n```\n\n```\n# test/my_app_test.exs\ndefmodule MyAppTest do\n  use ExUnit.Case, async: false\n  use Mimic\n\n  @tag :benchmark\n  test \"benchmarking async work!\" do\n    MyApp\n    |> stub(:do_work, fn _arg ->\n      Elixir.BencheeAsync.Reporter.report()\n      :ok\n    end)\n\n    # benchmarking code goes here...\n    BencheeAsync.run(%{....})\n  end\nend\n```","ref":"BencheeAsync.Reporter.html#module-reporting"},{"type":"function","title":"BencheeAsync.Reporter.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"BencheeAsync.Reporter.html#child_spec/1"},{"type":"function","title":"BencheeAsync.Reporter.clear/0","doc":"Resets the state within the Reporter. This is not called automatically after the benchmark is run, as it is expected that the Reporter process terminates after each benchmark run.\n\nIf this is not the case for you, you can run this manually between benchmark runs as so:\n\n```elixir\nBencheeAsync.run(%{...})\nBencheeAsync.Reporter.clear()\nBencheeAsync.run(%{...})\n```\n\nIf the scenario names are different between each benchmark run, then clearing the state would not be necessary.","ref":"BencheeAsync.Reporter.html#clear/0"},{"type":"function","title":"BencheeAsync.Reporter.get_samples/2","doc":"Retrieves the samples recorded for a given scenario and input combination.","ref":"BencheeAsync.Reporter.html#get_samples/2"},{"type":"function","title":"BencheeAsync.Reporter.record/0","doc":"Records a unit of work done. Should be called each time a unit of work is performed.\n\nIt is advised to mock the function that you wish to track and call this function within the mock.","ref":"BencheeAsync.Reporter.html#record/0"},{"type":"function","title":"BencheeAsync.Reporter.start_link/1","doc":"","ref":"BencheeAsync.Reporter.html#start_link/1"},{"type":"extras","title":"BencheeAsync","doc":"# BencheeAsync\n\n`Benchee` plugin for benchmarking multi-process performance for async work.\n\nThis plugin allows optimization of systems that are spread out over multiple processes. Benchee only allows benchmarking of a singular function within a singular executing process, and cannot keep track of cross-process work performed. **This plugin allows us to measure async units of work done, thereby allowing us to optimize our async pipelines.**\n\nThe goal of this library is to **approximately** track units of async work done and the rate of completion.","ref":"readme.html"},{"type":"extras","title":"Installation - BencheeAsync","doc":"```elixir\ndef deps do\n  [\n    # benchee is used internally\n    {:benchee, \"~> 1.0\", only: [:dev, :test]},\n    {:benchee_async, \"~> 0.1.0\", only: [:dev, :test]}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - BencheeAsync","doc":"The following **must be configured**:\n\n1. Start the `BencheeAsync.Reporter` GenServer.\n2. Benchmark functions must call `BencheeAsync.Reporter.record/0` to record a unit of work completed.\n3. Set the `extended_statistics: true` option for `Benchee.Formatters.Console`\n\n#","ref":"readme.html#usage"},{"type":"extras","title":"Example - BencheeAsync","doc":"This shows an example of running Benchee from within a ExUnit test suite.\n\n```elixir\ndefmodule MyAppTest do\n  use ExUnit.Case, async: false\n\n  test \"measure async work!\" do\n    # start the reporter process\n    start_supervised!(BencheeAsync.Reporter)\n\n    # use BencheeAsync instead of Benchee\n    BencheeAsync.run(\n      %{\n        \"case_100_ms\" => fn ->\n          Task.start(fn ->\n            :timer.sleep(100)\n            BencheeAsync.Reporter.record()\n          end)\n          :timer.sleep(2500)\n        end,\n        \"case_1000_ms\" => fn ->\n          Task.start(fn ->\n            :timer.sleep(1000)\n            BencheeAsync.Reporter.record()\n          end)\n          :timer.sleep(1500)\n        end\n      },\n      time: 1,\n      warmup: 3,\n      # use extended_statistics to view units of work done\n      formatters: [{Benchee.Formatters.Console, extended_statistics: true}]\n    )\n  end\nend\n```\n\nThe resulting console output will be as follows:\n\n```\nOperating System: macOS\nCPU Information: Apple M1 Pro\nNumber of Available Cores: 10\nAvailable memory: 32 GB\nElixir 1.15.5\nErlang 26.1.2\n\nBenchmark suite executing with the following configuration:\nwarmup: 3 s\ntime: 1 s\nmemory time: 0 ns\nreduction time: 0 ns\nparallel: 1\ninputs: none specified\nEstimated total run time: 8 s\n\nBenchmarking case_1000_ms ...\nBenchmarking case_100_ms ...\n\nName                       ips        average  deviation         median         99th %\ncase_100_ms           9.92        0.101 s     ±0.20%        0.101 s        0.101 s\ncase_1000_ms          1.00         1.00 s     ±0.04%         1.00 s         1.00 s\n\nComparison:\ncase_100_ms           9.92\ncase_1000_ms          1.00 - 9.93x slower +0.90 s\n\nExtended statistics:\n\nName                     minimum        maximum    sample size                     mode\ncase_100_ms          0.101 s        0.101 s              3                     None\ncase_1000_ms          1.00 s         1.00 s              1                     None\n```\n\nInterpretation differences from `Benchee` are as follows:\n\n- `ips`: The maximum iterations per second of the async process(es) if the async logic was repeatedly executed in isolation.\n- `average`, `deviation`, `median`, `99th %`: The statistics of execution time between each reported unit work done.\n- `sample size`: The amount of reported units of work done, which will correspond to the number of `BencheeAsync.Reporter.report/1` calls.\n\n#","ref":"readme.html#example"},{"type":"extras","title":"Usage with Inputs - BencheeAsync","doc":"Inputs work as well with no additional configuration needed.\n\n```\nOperating System: macOS\nCPU Information: Apple M1 Pro\nNumber of Available Cores: 10\nAvailable memory: 32 GB\nElixir 1.15.5\nErlang 26.1.2\n\nBenchmark suite executing with the following configuration:\nwarmup: 0 ns\ntime: 3 s\nmemory time: 0 ns\nreduction time: 0 ns\nparallel: 1\ninputs: Bigger, Medium, Small\nEstimated total run time: 18 s\n\nBenchmarking case_faster with input Bigger ...\nBenchmarking case_faster with input Medium ...\nBenchmarking case_faster with input Small ...\nBenchmarking case_slower with input Bigger ...\nBenchmarking case_slower with input Medium ...\nBenchmarking case_slower with input Small ...\n\n###","ref":"readme.html#usage-with-inputs"},{"type":"extras","title":"With input Bigger ##### - BencheeAsync","doc":"Name                  ips        average  deviation         median         99th %\ncase_faster        1.08 M     0.00092 ms    ±36.87%     0.00092 ms     0.00154 ms\ncase_slower     0.00001 M       75.90 ms     ±0.22%       75.94 ms       76.03 ms\n\nComparison:\ncase_faster        1.08 M\ncase_slower     0.00001 M - 82215.44x slower +75.90 ms\n\nExtended statistics:\n\nName                minimum        maximum    sample size                     mode\ncase_faster      0.00013 ms     0.00154 ms             39               0.00088 ms\ncase_slower        75.27 ms       76.03 ms             20                     None\n\n###","ref":"readme.html#with-input-bigger"},{"type":"extras","title":"With input Medium ##### - BencheeAsync","doc":"Name                  ips        average  deviation         median         99th %\ncase_faster      982.25 K     0.00102 ms   ±151.32%     0.00083 ms      0.0123 ms\ncase_slower      0.0196 K       51.04 ms     ±0.76%       51.00 ms       52.96 ms\n\nComparison:\ncase_faster      982.25 K\ncase_slower      0.0196 K - 50138.38x slower +51.04 ms\n\nExtended statistics:\n\nName                minimum        maximum    sample size                     mode\ncase_faster      0.00013 ms      0.0123 ms             58               0.00075 ms\ncase_slower        50.49 ms       52.96 ms             30                     None\n\n###","ref":"readme.html#with-input-medium"},{"type":"extras","title":"With input Small ##### - BencheeAsync","doc":"Name                  ips        average  deviation         median         99th %\ncase_faster        1.68 M     0.00059 ms    ±38.29%     0.00058 ms     0.00108 ms\ncase_slower     0.00009 M       11.00 ms     ±1.08%       11.01 ms       11.61 ms\n\nComparison:\ncase_faster        1.68 M\ncase_slower     0.00009 M - 18489.07x slower +11.00 ms\n\nExtended statistics:\n\nName                minimum        maximum    sample size                     mode\ncase_faster      0.00013 ms     0.00275 ms            272               0.00063 ms\ncase_slower        10.44 ms       11.69 ms            14311.02 ms, 11.04 ms, 11.01\n```\n\n#","ref":"readme.html#with-input-small"},{"type":"extras","title":"Usage in a Real World Application - BencheeAsync","doc":"It is advised to mock your async functions using [`:meck`](https://hexdocs.pm/meck/meck.html) or [`Mimic`](https://hexdocs.pm/mimic/Mimic.html). The mocked function would be where you trigger `BencheeAsync.Reporter.report/0`.\n\n#","ref":"readme.html#usage-in-a-real-world-application"},{"type":"extras","title":"Internals and Behavior - BencheeAsync","doc":"This library injects hooks into the `Benchee.run/1` in order to achieve async work benchmarking.\n\n`BencheeAsync` utilizes the `Benchee` public APIs only to achieve the hook injections. All user provided hooks will be executed **after** the injected hooks.\n\nGlobal hooks need to be injected in order to initiate tracking of post warmup timing and post-scenario timings.\n\nTo allow `BencheeAsync.Reporter.record/0` to work without specifying scenario name or input name, the input is used in the local `:before_scenario` hook in order to identify the scenario-input combination being benchmarked. The input is then hashed using `:erlang.phash2/2` for internal referencing.\n\n#","ref":"readme.html#internals-and-behavior"},{"type":"extras","title":"Limitations - BencheeAsync","doc":"The `memory_time` and `reduction_time` Benchee options will extend the execution time, hence the sample size will include counts beyond set run time value.","ref":"readme.html#limitations"}],"content_type":"text/markdown"}